@@ -16,11 +16,11 @@
 #
 
 from gettext import gettext as _
-import gconf
+from gi.repository import GConf
 
 
 def get_corner_delay():
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     corner_delay = client.get_int('/desktop/sugar/frame/corner_delay')
     return corner_delay
 
@@ -39,13 +39,13 @@ def set_corner_delay(delay):
         int(delay)
     except ValueError:
         raise ValueError(_('Value must be an integer.'))
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     client.set_int('/desktop/sugar/frame/corner_delay', int(delay))
     return 0
 
 
 def get_edge_delay():
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     edge_delay = client.get_int('/desktop/sugar/frame/edge_delay')
     return edge_delay
 
@@ -64,6 +64,6 @@ def set_edge_delay(delay):
         int(delay)
     except ValueError:
         raise ValueError(_('Value must be an integer.'))
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     client.set_int('/desktop/sugar/frame/edge_delay', int(delay))
     return 0
