@@ -15,12 +15,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gtk
-import gobject
+from gi.repository import Gtk
+from gi.repository import GObject
 import gettext
 
-from sugar.graphics import style
-from sugar.graphics.icon import Icon
+from sugar3.graphics import style
+from sugar3.graphics.icon import Icon
 
 from jarabe.controlpanel.sectionview import SectionView
 from jarabe.controlpanel.inlinealert import InlineAlert
@@ -57,28 +57,27 @@ class Language(SectionView):
         explanation = gettext.gettext('Add languages in the order you prefer.'
                                       ' If a translation is not available,'
                                       ' the next in the list will be used.')
-        self._text = gtk.Label(explanation)
-        self._text.set_width_chars(100)
+        self._text = Gtk.Label(label=explanation)
         self._text.set_line_wrap(True)
         self._text.set_alignment(0, 0)
-        self.pack_start(self._text, False)
+        self.pack_start(self._text, False, False, 0)
         self._text.show()
 
-        scrolled = gtk.ScrolledWindow()
-        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        scrolled = Gtk.ScrolledWindow()
+        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         scrolled.show()
-        self.pack_start(scrolled, expand=True)
+        self.pack_start(scrolled, True, True, 0)
 
-        self._table = gtk.Table(rows=1, columns=3, homogeneous=False)
+        self._table = Gtk.Table(rows=1, columns=3, homogeneous=False)
         self._table.set_border_width(style.DEFAULT_SPACING * 2)
         self._table.show()
         scrolled.add_with_viewport(self._table)
 
-        self._lang_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
-        self.pack_start(self._lang_alert_box, False)
+        self._lang_alert_box = Gtk.HBox(spacing=style.DEFAULT_SPACING)
+        self.pack_start(self._lang_alert_box, False, True, 0)
 
         self._lang_alert = InlineAlert()
-        self._lang_alert_box.pack_start(self._lang_alert)
+        self._lang_alert_box.pack_start(self._lang_alert, True, True, 0)
         if 'lang' in self.restart_alerts:
             self._lang_alert.props.msg = self.restart_msg
             self._lang_alert.show()
@@ -93,22 +92,22 @@ class Language(SectionView):
 
         self._table.resize(self._selected_lang_count, 3)
 
-        label = gtk.Label(str=str(self._selected_lang_count))
-        label.modify_fg(gtk.STATE_NORMAL,
+        label = Gtk.Label(label=str(self._selected_lang_count))
+        label.modify_fg(Gtk.StateType.NORMAL,
             style.COLOR_SELECTION_GREY.get_gdk_color())
         self._labels.append(label)
         self._attach_to_table(label, 0, 1, padding=1)
         label.show()
 
-        store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
+        store = Gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_STRING)
         for language, country, code in self._available_locales:
             description = '%s (%s)' % (_translate_language(language), \
                 _translate_country(country))
             store.append([code, description])
 
-        combobox = gtk.ComboBox(model=store)
-        cell = gtk.CellRendererText()
-        combobox.pack_start(cell)
+        combobox = Gtk.ComboBox(model=store)
+        cell = Gtk.CellRendererText()
+        combobox.pack_start(cell, True)
         combobox.add_attribute(cell, 'text', 1)
 
         if locale_code:
@@ -125,7 +124,7 @@ class Language(SectionView):
 
         self._stores.append(store)
         self._comboboxes.append(combobox)
-        self._attach_to_table(combobox, 1, 2, yoptions=gtk.SHRINK)
+        self._attach_to_table(combobox, 1, 2, yoptions=Gtk.AttachOptions.SHRINK)
 
         add_remove_box = self._create_add_remove_box()
         self._add_remove_boxes.append(add_remove_box)
@@ -135,17 +134,21 @@ class Language(SectionView):
 
         if self._selected_lang_count > 1:
             previous_add_removes = self._add_remove_boxes[-2]
-            previous_add_removes.hide_all()
+            previous_add_removes.hide()
 
-        self._determine_add_remove_visibility()
+        # Hide the Remove button if the new added row is the only
+        # language.
+        elif self._selected_lang_count == 1:
+            add_button_, remove_button = add_remove_box.get_children()
+            remove_button.props.visible = False
 
         combobox.show()
 
     def _attach_to_table(self, widget, row, column, padding=20, \
-            yoptions=gtk.FILL):
+            yoptions=Gtk.AttachOptions.FILL):
         self._table.attach(widget, row, column, \
             self._selected_lang_count - 1, self._selected_lang_count, \
-            xoptions=gtk.FILL, yoptions=yoptions, xpadding=padding, \
+            xoptions=Gtk.AttachOptions.FILL, yoptions=yoptions, xpadding=padding, \
                 ypadding=padding)
 
     def _delete_last_row(self):
@@ -163,6 +166,15 @@ class Language(SectionView):
 
         self._add_remove_boxes[-1].show_all()
 
+        # Hide or show the Remove button in the new last row,
+        # depending if it is the only language.
+        add_remove_box = self._add_remove_boxes[-1]
+        add_button_, remove_button = add_remove_box.get_children()
+        if self._selected_lang_count == 1:
+            remove_button.props.visible = False
+        else:
+            remove_button.props.visible = True
+
     def _get_last_row(self):
         label = self._labels.pop()
         add_remove_box = self._add_remove_boxes.pop()
@@ -180,25 +192,25 @@ class Language(SectionView):
         self._lang_alert.hide()
 
     def _create_add_remove_box(self):
-        """Creates gtk.Hbox with add/remove buttons"""
+        """Creates Gtk.Hbox with add/remove buttons"""
         add_icon = Icon(icon_name='list-add')
 
-        add_button = gtk.Button()
+        add_button = Gtk.Button()
         add_button.set_image(add_icon)
         add_button.connect('clicked',
                             self.__add_button_clicked_cb)
 
         remove_icon = Icon(icon_name='list-remove')
-        remove_button = gtk.Button()
+        remove_button = Gtk.Button()
         remove_button.set_image(remove_icon)
         remove_button.connect('clicked',
                             self.__remove_button_clicked_cb)
 
-        add_remove_box = gtk.HButtonBox()
-        add_remove_box.set_layout(gtk.BUTTONBOX_START)
+        add_remove_box = Gtk.HButtonBox()
+        add_remove_box.set_layout(Gtk.ButtonBoxStyle.START)
         add_remove_box.set_spacing(10)
-        add_remove_box.pack_start(add_button)
-        add_remove_box.pack_start(remove_button)
+        add_remove_box.pack_start(add_button, True, True, 0)
+        add_remove_box.pack_start(remove_button, True, True, 0)
 
         return add_remove_box
 
@@ -217,8 +229,6 @@ class Language(SectionView):
         selected_langs = self._get_selected_langs()
         last_lang = selected_langs[-1]
 
-        self._determine_add_remove_visibility(last_lang=last_lang)
-
         self._changed = (selected_langs != self._selected_locales)
 
         if self._changed == False:
@@ -228,13 +238,13 @@ class Language(SectionView):
                 self.restart_alerts.remove('lang')
             self._lang_alert.hide()
             if self._lang_sid:
-                gobject.source_remove(self._lang_sid)
+                GObject.source_remove(self._lang_sid)
             self._model.undo()
             return False
 
         if self._lang_sid:
-            gobject.source_remove(self._lang_sid)
-        self._lang_sid = gobject.timeout_add(self._APPLY_TIMEOUT,
+            GObject.source_remove(self._lang_sid)
+        self._lang_sid = GObject.timeout_add(self._APPLY_TIMEOUT,
                                             self.__lang_timeout_cb,
                                             selected_langs)
 
@@ -248,29 +258,6 @@ class Language(SectionView):
 
         return new_codes
 
-    def _determine_add_remove_visibility(self, last_lang=None):
-        # We should not let users add fallback languages for English (USA)
-        # This is because the software is not usually _translated_ into English
-        # which means that the fallback gets selected automatically
-
-        if last_lang is None:
-            selected_langs = self._get_selected_langs()
-            last_lang = selected_langs[-1]
-
-        add_remove_box = self._add_remove_boxes[-1]
-        buttons = add_remove_box.get_children()
-        add_button, remove_button = buttons
-
-        if last_lang.startswith('en_US'):
-            add_button.props.visible = False
-        else:
-            add_button.props.visible = True
-
-        if self._selected_lang_count == 1:
-            remove_button.props.visible = False
-        else:
-            remove_button.props.visible = True
-
     def __lang_timeout_cb(self, codes):
         self._lang_sid = 0
         self._model.set_languages_list(codes)
