@@ -19,7 +19,7 @@ import logging
 
 import dbus
 from gettext import gettext as _
-import gconf
+from gi.repository import GConf
 
 from jarabe.model import network
 
@@ -40,7 +40,7 @@ class ReadError(Exception):
 
 
 def get_jabber():
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     return client.get_string('/desktop/sugar/collaboration/jabber_server')
 
 
@@ -52,7 +52,7 @@ def set_jabber(server):
     """Set the jabber server
     server : e.g. 'olpc.collabora.co.uk'
     """
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     client.set_string('/desktop/sugar/collaboration/jabber_server', server)
 
     return 0
@@ -106,7 +106,7 @@ def set_radio(state):
 def clear_registration():
     """Clear the registration with the schoolserver
     """
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     client.set_string('/desktop/sugar/backup_url', '')
     return 1
 
@@ -132,7 +132,7 @@ def have_networks():
 
 
 def get_publish_information():
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     publish = client.get_bool('/desktop/sugar/collaboration/publish_gadget')
     return publish
 
@@ -151,6 +151,6 @@ def set_publish_information(value):
     except:
         raise ValueError(_('Error in specified argument use 0/1.'))
 
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
     client.set_bool('/desktop/sugar/collaboration/publish_gadget', value)
     return 0
