@@ -19,40 +19,45 @@ import os
 import logging
 from gettext import gettext as _
 
-import gobject
-import pango
-import gconf
-import gtk
+from gi.repository import GLib
+from gi.repository import GObject
+from gi.repository import Pango
+from gi.repository import GConf
+from gi.repository import Gtk
+from gi.repository import Gdk
 
-from sugar import util
-from sugar.graphics import style
-from sugar.graphics.icon import Icon, CellRendererIcon
-from sugar.graphics.xocolor import XoColor
-from sugar.graphics.menuitem import MenuItem
-from sugar.graphics.alert import Alert
+from sugar3 import util
+from sugar3.graphics import style
+from sugar3.graphics.icon import Icon, CellRendererIcon
+from sugar3.graphics.xocolor import XoColor
+from sugar3.graphics.alert import Alert
+from sugar3.graphics.palettemenu import PaletteMenuItem
 
 from jarabe.model import bundleregistry
 from jarabe.view.palettes import ActivityPalette
 from jarabe.journal import misc
+from jarabe.util.normalize import normalize_string
 
 
-class ActivitiesTreeView(gtk.TreeView):
+class ActivitiesTreeView(Gtk.TreeView):
     __gtype_name__ = 'SugarActivitiesTreeView'
 
     __gsignals__ = {
-        'erase-activated': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+        'erase-activated': (GObject.SignalFlags.RUN_FIRST, None,
                             ([str])),
     }
 
     def __init__(self):
-        gobject.GObject.__init__(self)
+        Gtk.TreeView.__init__(self)
 
         self._query = ''
 
-        self.modify_base(gtk.STATE_NORMAL, style.COLOR_WHITE.get_gdk_color())
         self.set_headers_visible(False)
+        self.add_events(Gdk.EventMask.BUTTON_PRESS_MASK |
+                        Gdk.EventMask.TOUCH_MASK |
+                        Gdk.EventMask.BUTTON_RELEASE_MASK)
         selection = self.get_selection()
-        selection.set_mode(gtk.SELECTION_NONE)
+        selection.set_mode(Gtk.SelectionMode.NONE)
 
         model = ListModel()
         model.set_visible_func(self.__model_visible_cb)
@@ -61,8 +66,8 @@ class ActivitiesTreeView(gtk.TreeView):
         cell_favorite = CellRendererFavorite(self)
         cell_favorite.connect('clicked', self.__favorite_clicked_cb)
 
-        column = gtk.TreeViewColumn()
-        column.pack_start(cell_favorite)
+        column = Gtk.TreeViewColumn()
+        column.pack_start(cell_favorite, True)
         column.set_cell_data_func(cell_favorite, self.__favorite_set_data_cb)
         self.append_column(column)
 
@@ -70,48 +75,48 @@ class ActivitiesTreeView(gtk.TreeView):
         cell_icon.connect('erase-activated', self.__erase_activated_cb)
         cell_icon.connect('clicked', self.__icon_clicked_cb)
 
-        column = gtk.TreeViewColumn()
-        column.pack_start(cell_icon)
+        column = Gtk.TreeViewColumn()
+        column.pack_start(cell_icon, True)
         column.add_attribute(cell_icon, 'file-name', ListModel.COLUMN_ICON)
         self.append_column(column)
 
-        cell_text = gtk.CellRendererText()
-        cell_text.props.ellipsize = pango.ELLIPSIZE_MIDDLE
+        cell_text = Gtk.CellRendererText()
+        cell_text.props.ellipsize = Pango.EllipsizeMode.MIDDLE
         cell_text.props.ellipsize_set = True
 
-        column = gtk.TreeViewColumn()
-        column.props.sizing = gtk.TREE_VIEW_COLUMN_GROW_ONLY
+        column = Gtk.TreeViewColumn()
+        column.props.sizing = Gtk.TreeViewColumnSizing.GROW_ONLY
         column.props.expand = True
         column.set_sort_column_id(ListModel.COLUMN_TITLE)
-        column.pack_start(cell_text)
+        column.pack_start(cell_text, True)
         column.add_attribute(cell_text, 'markup', ListModel.COLUMN_TITLE)
         self.append_column(column)
 
-        cell_text = gtk.CellRendererText()
+        cell_text = Gtk.CellRendererText()
         cell_text.props.xalign = 1
 
-        column = gtk.TreeViewColumn()
+        column = Gtk.TreeViewColumn()
         column.set_alignment(1)
-        column.props.sizing = gtk.TREE_VIEW_COLUMN_GROW_ONLY
+        column.props.sizing = Gtk.TreeViewColumnSizing.GROW_ONLY
         column.props.resizable = True
         column.props.reorderable = True
         column.props.expand = True
         column.set_sort_column_id(ListModel.COLUMN_VERSION)
-        column.pack_start(cell_text)
+        column.pack_start(cell_text, True)
         column.add_attribute(cell_text, 'text', ListModel.COLUMN_VERSION_TEXT)
         self.append_column(column)
 
-        cell_text = gtk.CellRendererText()
+        cell_text = Gtk.CellRendererText()
         cell_text.props.xalign = 1
 
-        column = gtk.TreeViewColumn()
+        column = Gtk.TreeViewColumn()
         column.set_alignment(1)
-        column.props.sizing = gtk.TREE_VIEW_COLUMN_GROW_ONLY
+        column.props.sizing = Gtk.TreeViewColumnSizing.GROW_ONLY
         column.props.resizable = True
         column.props.reorderable = True
         column.props.expand = True
         column.set_sort_column_id(ListModel.COLUMN_DATE)
-        column.pack_start(cell_text)
+        column.pack_start(cell_text, True)
         column.add_attribute(cell_text, 'text', ListModel.COLUMN_DATE_TEXT)
         self.append_column(column)
 
@@ -121,10 +126,10 @@ class ActivitiesTreeView(gtk.TreeView):
     def __erase_activated_cb(self, cell_renderer, bundle_id):
         self.emit('erase-activated', bundle_id)
 
-    def __favorite_set_data_cb(self, column, cell, model, tree_iter):
+    def __favorite_set_data_cb(self, column, cell, model, tree_iter, data):
         favorite = model[tree_iter][ListModel.COLUMN_FAVORITE]
         if favorite:
-            client = gconf.client_get_default()
+            client = GConf.Client.get_default()
             color = XoColor(client.get_string('/desktop/sugar/user/color'))
             cell.props.xo_color = color
         else:
@@ -146,15 +151,22 @@ class ActivitiesTreeView(gtk.TreeView):
         misc.launch(bundle)
 
     def set_filter(self, query):
-        self._query = query.lower()
+        """Set a new query and refilter the model, return the number
+        of matching activities.
+
+        """
+        self._query = normalize_string(query.decode('utf-8'))
         self.get_model().refilter()
+        matches = self.get_model().iter_n_children(None)
+        return matches
 
-    def __model_visible_cb(self, model, tree_iter):
+    def __model_visible_cb(self, model, tree_iter, data):
         title = model[tree_iter][ListModel.COLUMN_TITLE]
-        return title is not None and title.lower().find(self._query) > -1
+        title = normalize_string(title.decode('utf-8'))
+        return title is not None and title.find(self._query) > -1
 
 
-class ListModel(gtk.TreeModelSort):
+class ListModel(Gtk.TreeModelSort):
     __gtype_name__ = 'SugarListModel'
 
     COLUMN_BUNDLE_ID = 0
@@ -167,11 +179,12 @@ class ListModel(gtk.TreeModelSort):
     COLUMN_DATE_TEXT = 7
 
     def __init__(self):
-        self._model = gtk.ListStore(str, bool, str, str, str, str, int, str)
+        self._model = Gtk.ListStore(str, bool, str, str, str, str, int, str)
         self._model_filter = self._model.filter_new()
-        gtk.TreeModelSort.__init__(self, self._model_filter)
+        Gtk.TreeModelSort.__init__(self, model=self._model_filter)
+        self.set_sort_column_id(ListModel.COLUMN_TITLE, Gtk.SortType.ASCENDING)
 
-        gobject.idle_add(self.__connect_to_bundle_registry_cb)
+        GObject.idle_add(self.__connect_to_bundle_registry_cb)
 
     def __connect_to_bundle_registry_cb(self):
         registry = bundleregistry.get_registry()
@@ -229,7 +242,7 @@ class ListModel(gtk.TreeModelSort):
                             title,
                             version,
                             _('Version %s') % version,
-                            timestamp,
+                            int(timestamp),
                             util.timestamp_to_elapsed_string(timestamp)])
 
     def set_visible_func(self, func):
@@ -249,8 +262,8 @@ class CellRendererFavorite(CellRendererIcon):
         self.props.height = style.GRID_CELL_SIZE
         self.props.size = style.SMALL_ICON_SIZE
         self.props.icon_name = 'emblem-favorite'
-        self.props.mode = gtk.CELL_RENDERER_MODE_ACTIVATABLE
-        client = gconf.client_get_default()
+        self.props.mode = Gtk.CellRendererMode.ACTIVATABLE
+        client = GConf.Client.get_default()
         prelit_color = XoColor(client.get_string('/desktop/sugar/user/color'))
         self.props.prelit_stroke_color = prelit_color.get_stroke_color()
         self.props.prelit_fill_color = prelit_color.get_fill_color()
@@ -260,7 +273,7 @@ class CellRendererActivityIcon(CellRendererIcon):
     __gtype_name__ = 'SugarCellRendererActivityIcon'
 
     __gsignals__ = {
-        'erase-activated': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+        'erase-activated': (GObject.SignalFlags.RUN_FIRST, None,
                             ([str])),
     }
 
@@ -272,9 +285,9 @@ class CellRendererActivityIcon(CellRendererIcon):
         self.props.size = style.STANDARD_ICON_SIZE
         self.props.stroke_color = style.COLOR_BUTTON_GREY.get_svg()
         self.props.fill_color = style.COLOR_TRANSPARENT.get_svg()
-        self.props.mode = gtk.CELL_RENDERER_MODE_ACTIVATABLE
+        self.props.mode = Gtk.CellRendererMode.ACTIVATABLE
 
-        client = gconf.client_get_default()
+        client = GConf.Client.get_default()
         prelit_color = XoColor(client.get_string('/desktop/sugar/user/color'))
         self.props.prelit_stroke_color = prelit_color.get_stroke_color()
         self.props.prelit_fill_color = prelit_color.get_fill_color()
@@ -295,33 +308,90 @@ class CellRendererActivityIcon(CellRendererIcon):
         self.emit('erase-activated', bundle_id)
 
 
-class ActivitiesList(gtk.VBox):
+class ClearMessageBox(Gtk.EventBox):
+    def __init__(self, message, button_callback):
+        Gtk.EventBox.__init__(self)
+
+        self.modify_bg(Gtk.StateType.NORMAL,
+                       style.COLOR_WHITE.get_gdk_color())
+
+        alignment = Gtk.Alignment.new(0.5, 0.5, 0.1, 0.1)
+        self.add(alignment)
+        alignment.show()
+
+        box = Gtk.VBox()
+        alignment.add(box)
+        box.show()
+
+        icon = Icon(pixel_size=style.LARGE_ICON_SIZE,
+                    icon_name='system-search',
+                    stroke_color=style.COLOR_TRANSPARENT.get_svg(),
+                    fill_color=style.COLOR_BUTTON_GREY.get_svg())
+        box.pack_start(icon, expand=True, fill=False, padding=0)
+        icon.show()
+
+        label = Gtk.Label()
+        color = style.COLOR_BUTTON_GREY.get_html()
+        label.set_markup('<span weight="bold" color="%s">%s</span>' % ( \
+                color, GLib.markup_escape_text(message)))
+        box.pack_start(label, expand=True, fill=False, padding=0)
+        label.show()
+
+        button_box = Gtk.HButtonBox()
+        button_box.set_layout(Gtk.ButtonBoxStyle.CENTER)
+        box.pack_start(button_box, False, True, 0)
+        button_box.show()
+
+        button = Gtk.Button(label=_('Clear search'))
+        button.connect('clicked', button_callback)
+        button.props.image = Icon(icon_name='dialog-cancel',
+                                  icon_size=Gtk.IconSize.BUTTON)
+        button_box.pack_start(button, expand=True, fill=False, padding=0)
+        button.show()
+
+
+class ActivitiesList(Gtk.VBox):
     __gtype_name__ = 'SugarActivitiesList'
 
+    __gsignals__ = {
+        'clear-clicked': (GObject.SignalFlags.RUN_FIRST, None, ([])),
+    }
+
     def __init__(self):
         logging.debug('STARTUP: Loading the activities list')
 
-        gobject.GObject.__init__(self)
+        Gtk.VBox.__init__(self)
 
-        scrolled_window = gtk.ScrolledWindow()
-        scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        scrolled_window.set_shadow_type(gtk.SHADOW_NONE)
-        scrolled_window.connect('key-press-event', self.__key_press_event_cb)
-        self.pack_start(scrolled_window)
-        scrolled_window.show()
+        self._scrolled_window = Gtk.ScrolledWindow()
+        self._scrolled_window.set_policy(Gtk.PolicyType.NEVER,
+                                         Gtk.PolicyType.AUTOMATIC)
+        self._scrolled_window.set_shadow_type(Gtk.ShadowType.NONE)
+        self._scrolled_window.connect('key-press-event',
+                                      self.__key_press_event_cb)
+        self.pack_start(self._scrolled_window, True, True, 0)
+        self._scrolled_window.show()
 
         self._tree_view = ActivitiesTreeView()
         self._tree_view.connect('erase-activated', self.__erase_activated_cb)
-        scrolled_window.add(self._tree_view)
+        self._scrolled_window.add(self._tree_view)
         self._tree_view.show()
 
         self._alert = None
+        self._clear_message_box = None
+
+    def grab_focus(self):
+        # overwrite grab focus in order to grab focus from the parent
+        self._tree_view.grab_focus()
 
     def set_filter(self, query):
-        self._tree_view.set_filter(query)
+        matches = self._tree_view.set_filter(query)
+        if matches == 0:
+            self._show_clear_message()
+        else:
+            self._hide_clear_message()
 
     def __key_press_event_cb(self, scrolled_window, event):
-        keyname = gtk.gdk.keyval_name(event.keyval)
+        keyname = Gdk.keyval_name(event.keyval)
 
         vadjustment = scrolled_window.props.vadjustment
         if keyname == 'Up':
@@ -338,11 +408,38 @@ class ActivitiesList(gtk.VBox):
 
         return True
 
+    def _show_clear_message(self):
+        if self._clear_message_box in self.get_children():
+            return
+        if self._scrolled_window in self.get_children():
+            self.remove(self._scrolled_window)
+
+        self._clear_message_box = ClearMessageBox(
+            message=_('No matching activities'),
+            button_callback=self.__clear_button_clicked_cb)
+
+        self.pack_end(self._clear_message_box, True, True, 0)
+        self._clear_message_box.show()
+
+    def __clear_button_clicked_cb(self, button):
+        self.emit('clear-clicked')
+
+    def _hide_clear_message(self):
+        if self._scrolled_window in self.get_children():
+            return
+        if self._clear_message_box in self.get_children():
+            self.remove(self._clear_message_box)
+
+        self._clear_message_box = None
+
+        self.pack_end(self._scrolled_window, True, True, 0)
+        self._scrolled_window.show()
+
     def add_alert(self, alert):
         if self._alert is not None:
             self.remove_alert()
         self._alert = alert
-        self.pack_start(alert, False)
+        self.pack_start(alert, False, True, 0)
         self.reorder_child(alert, 0)
 
     def remove_alert(self):
@@ -360,10 +457,10 @@ class ActivitiesList(gtk.VBox):
                 % activity_info.get_name()
 
         cancel_icon = Icon(icon_name='dialog-cancel')
-        alert.add_button(gtk.RESPONSE_CANCEL, _('Keep'), cancel_icon)
+        alert.add_button(Gtk.ResponseType.CANCEL, _('Keep'), cancel_icon)
 
         erase_icon = Icon(icon_name='dialog-ok')
-        alert.add_button(gtk.RESPONSE_OK, _('Erase'), erase_icon)
+        alert.add_button(Gtk.ResponseType.OK, _('Erase'), erase_icon)
 
         alert.connect('response', self.__erase_confirmation_dialog_response_cb,
                 bundle_id)
@@ -373,7 +470,7 @@ class ActivitiesList(gtk.VBox):
     def __erase_confirmation_dialog_response_cb(self, alert, response_id,
                                                 bundle_id):
         self.remove_alert()
-        if response_id == gtk.RESPONSE_OK:
+        if response_id == Gtk.ResponseType.OK:
             registry = bundleregistry.get_registry()
             bundle = registry.get_bundle(bundle_id)
             registry.uninstall(bundle, delete_profile=True)
@@ -383,7 +480,7 @@ class ActivityListPalette(ActivityPalette):
     __gtype_name__ = 'SugarActivityListPalette'
 
     __gsignals__ = {
-        'erase-activated': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+        'erase-activated': (GObject.SignalFlags.RUN_FIRST, None,
                             ([str])),
     }
 
@@ -397,13 +494,14 @@ class ActivityListPalette(ActivityPalette):
         self._favorite = registry.is_bundle_favorite(self._bundle_id,
                                                      self._version)
 
-        self._favorite_item = MenuItem('')
+        self._favorite_item = PaletteMenuItem()
         self._favorite_icon = Icon(icon_name='emblem-favorite',
-                icon_size=gtk.ICON_SIZE_MENU)
+                icon_size=Gtk.IconSize.MENU)
         self._favorite_item.set_image(self._favorite_icon)
+        self._favorite_icon.show()
         self._favorite_item.connect('activate',
                                     self.__change_favorite_activate_cb)
-        self.menu.append(self._favorite_item)
+        self.menu_box.append_item(self._favorite_item)
         self._favorite_item.show()
 
         if activity_info.is_user_activity():
@@ -414,12 +512,12 @@ class ActivityListPalette(ActivityPalette):
                 self.__activity_changed_cb)
         self._update_favorite_item()
 
-        self.connect('destroy', self.__destroy_cb)
+        self.menu_box.connect('destroy', self.__destroy_cb)
 
     def _add_erase_option(self, registry, activity_info):
-        menu_item = MenuItem(_('Erase'), 'list-remove')
+        menu_item = PaletteMenuItem(_('Erase'), 'list-remove')
         menu_item.connect('activate', self.__erase_activate_cb)
-        self.menu.append(menu_item)
+        self.menu_box.append_item(menu_item)
         menu_item.show()
 
         if not os.access(activity_info.get_path(), os.W_OK) or \
@@ -431,14 +529,13 @@ class ActivityListPalette(ActivityPalette):
         registry.disconnect(self._activity_changed_sid)
 
     def _update_favorite_item(self):
-        label = self._favorite_item.child
         if self._favorite:
-            label.set_text(_('Remove favorite'))
+            self._favorite_item.set_label(_('Remove favorite'))
             xo_color = XoColor('%s,%s' % (style.COLOR_WHITE.get_svg(),
                                          style.COLOR_TRANSPARENT.get_svg()))
         else:
-            label.set_text(_('Make favorite'))
-            client = gconf.client_get_default()
+            self._favorite_item.set_label(_('Make favorite'))
+            client = GConf.Client.get_default()
             xo_color = XoColor(client.get_string('/desktop/sugar/user/color'))
 
         self._favorite_icon.props.xo_color = xo_color
