@@ -19,23 +19,28 @@ import logging
 from gettext import gettext as _
 import math
 
-import gobject
-import gconf
-import glib
-import gtk
-import hippo
+from gi.repository import GObject
+from gi.repository import GConf
+from gi.repository import GLib
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GdkPixbuf
 
-from sugar.graphics import style
-from sugar.graphics.icon import Icon, CanvasIcon
-from sugar.graphics.menuitem import MenuItem
-from sugar.graphics.alert import Alert
-from sugar.graphics.xocolor import XoColor
-from sugar.activity import activityfactory
-from sugar import dispatch
-from sugar.datastore import datastore
+from sugar3.graphics import style
+from sugar3.graphics.icon import Icon
+from sugar3.graphics.icon import CanvasIcon
+from sugar3.graphics.palettemenu import PaletteMenuBox
+from sugar3.graphics.palettemenu import PaletteMenuItem
+from sugar3.graphics.palettemenu import PaletteMenuItemSeparator
+from sugar3.graphics.alert import Alert
+from sugar3.graphics.xocolor import XoColor
+from sugar3.activity import activityfactory
+from sugar3 import dispatch
+from sugar3.datastore import datastore
 
 from jarabe.view.palettes import JournalPalette
-from jarabe.view.palettes import CurrentActivityPalette, ActivityPalette
+from jarabe.view.palettes import CurrentActivityPalette
+from jarabe.view.palettes import ActivityPalette
 from jarabe.view.buddyicon import BuddyIcon
 from jarabe.view.buddymenu import BuddyMenu
 from jarabe.model.buddy import get_owner_instance
@@ -46,11 +51,13 @@ from jarabe.journal import misc
 from jarabe.desktop import schoolserver
 from jarabe.desktop.schoolserver import RegisterError
 from jarabe.desktop import favoriteslayout
+from jarabe.desktop.viewcontainer import ViewContainer
+from jarabe.util.normalize import normalize_string
 
 
 _logger = logging.getLogger('FavoritesView')
 
-_ICON_DND_TARGET = ('activity-icon', gtk.TARGET_SAME_WIDGET, 0)
+_ICON_DND_TARGET = ('activity-icon', Gtk.TargetFlags.SAME_WIDGET, 0)
 
 LAYOUT_MAP = {favoriteslayout.RingLayout.key: favoriteslayout.RingLayout,
         #favoriteslayout.BoxLayout.key: favoriteslayout.BoxLayout,
@@ -64,66 +71,219 @@ about the layout can be accessed with fields of the class."""
 _favorites_settings = None
 
 
-class FavoritesView(hippo.Canvas):
+class FavoritesBox(Gtk.VBox):
+    __gtype_name__ = 'SugarFavoritesBox'
+
+    def __init__(self):
+        Gtk.VBox.__init__(self)
+
+        self._view = FavoritesView(self)
+        self.pack_start(self._view, True, True, 0)
+        self._view.show()
+
+        self._alert = None
+
+    def set_filter(self, query):
+        self._view.set_filter(query)
+
+    def set_resume_mode(self, resume_mode):
+        self._view.set_resume_mode(resume_mode)
+
+    def grab_focus(self):
+        # overwrite grab focus in order to grab focus from the parent
+        self._view.grab_focus()
+
+    def add_alert(self, alert):
+        if self._alert is not None:
+            self.remove_alert()
+        self._alert = alert
+        self.pack_start(alert, False, True, 0)
+        self.reorder_child(alert, 0)
+
+    def remove_alert(self):
+        self.remove(self._alert)
+        self._alert = None
+
+
+class FavoritesView(ViewContainer):
     __gtype_name__ = 'SugarFavoritesView'
 
-    def __init__(self, **kwargs):
-        logging.debug('STARTUP: Loading the favorites view')
+    def __init__(self, box):
+        self._box = box
+        self._layout = None
 
-        gobject.GObject.__init__(self, **kwargs)
+        owner_icon = OwnerIcon(style.XLARGE_ICON_SIZE)
+        owner_icon.connect('register-activate', self.__register_activate_cb)
 
-        # DND stuff
+        current_activity = CurrentActivityIcon()
+
+        ViewContainer.__init__(self, layout=self._layout,
+                               owner_icon=owner_icon,
+                               activity_icon=current_activity)
+
+        self.add_events(Gdk.EventMask.BUTTON_PRESS_MASK |
+                        Gdk.EventMask.POINTER_MOTION_HINT_MASK)
+        self.drag_dest_set(0, [], 0)
+
+        # Drag and drop is set only for the Random layout.  This is
+        # the flag that enables or disables it.
+        self._dragging_mode = False
+
+        self._drag_motion_hid = None
+        self._drag_drop_hid = None
+        self._drag_data_received_hid = None
+
+        self._dragging = False
         self._pressed_button = None
-        self._press_start_x = None
-        self._press_start_y = None
+        self._press_start_x = 0
+        self._press_start_y = 0
         self._hot_x = None
         self._hot_y = None
         self._last_clicked_icon = None
 
-        self._box = hippo.CanvasBox()
-        self._box.props.background_color = style.COLOR_WHITE.get_int()
-        self.set_root(self._box)
-
-        self._my_icon = OwnerIcon(style.XLARGE_ICON_SIZE)
-        self._my_icon.connect('register-activate', self.__register_activate_cb)
-        self._box.append(self._my_icon)
-
-        self._current_activity = CurrentActivityIcon()
-        self._box.append(self._current_activity)
-
-        self._layout = None
         self._alert = None
         self._resume_mode = True
 
-        # More DND stuff
-        self.add_events(gtk.gdk.BUTTON_PRESS_MASK |
-                        gtk.gdk.POINTER_MOTION_HINT_MASK)
-        self.connect('motion-notify-event', self.__motion_notify_event_cb)
-        self.connect('button-press-event', self.__button_press_event_cb)
-        self.connect('drag-begin', self.__drag_begin_cb)
-        self.connect('drag-motion', self.__drag_motion_cb)
-        self.connect('drag-drop', self.__drag_drop_cb)
-        self.connect('drag-data-received', self.__drag_data_received_cb)
-
-        gobject.idle_add(self.__connect_to_bundle_registry_cb)
+        GObject.idle_add(self.__connect_to_bundle_registry_cb)
 
         favorites_settings = get_settings()
         favorites_settings.changed.connect(self.__settings_changed_cb)
         self._set_layout(favorites_settings.layout)
 
-    def set_filter(self, query):
-        query = query.strip()
-        for icon in self._box.get_children():
-            if icon not in [self._my_icon, self._current_activity]:
-                activity_name = icon.get_activity_name().lower()
-                if activity_name.find(query) > -1:
-                    icon.alpha = 1.0
-                else:
-                    icon.alpha = 0.33
-
     def __settings_changed_cb(self, **kwargs):
         favorites_settings = get_settings()
-        self._set_layout(favorites_settings.layout)
+        layout_set = self._set_layout(favorites_settings.layout)
+        if layout_set:
+            self.set_layout(self._layout)
+            registry = bundleregistry.get_registry()
+            for info in registry:
+                if registry.is_bundle_favorite(info.get_bundle_id(),
+                                               info.get_activity_version()):
+                    self._add_activity(info)
+
+    def _set_layout(self, layout):
+        if layout not in LAYOUT_MAP:
+            logging.warn('Unknown favorites layout: %r', layout)
+            layout = favoriteslayout.RingLayout.key
+            assert layout in LAYOUT_MAP
+
+        if type(self._layout) == LAYOUT_MAP[layout]:
+            return False
+
+        if self._layout is not None and self._dragging_mode:
+            self.disconnect(self._drag_motion_hid)
+            self.disconnect(self._drag_drop_hid)
+            self.disconnect(self._drag_data_received_hid)
+
+        if layout == favoriteslayout.RandomLayout.key:
+            self._dragging_mode = True
+            self._drag_motion_hid = self.connect(
+                'drag-motion', self.__drag_motion_cb)
+            self._drag_drop_hid = self.connect(
+                'drag-drop', self.__drag_drop_cb)
+            self._drag_data_received_hid = self.connect(
+                'drag-data-received', self.__drag_data_received_cb)
+        else:
+            self._dragging_mode = False
+
+        self._layout = LAYOUT_MAP[layout]()
+        return True
+
+    layout = property(None, _set_layout)
+
+    def do_add(self, child):
+        if child != self._owner_icon and child != self._activity_icon:
+            self._children.append(child)
+            child.connect('button-press-event', self.__button_press_cb)
+            child.connect('button-release-event', self.__button_release_cb)
+            child.connect('motion-notify-event', self.__motion_notify_event_cb)
+            child.connect('drag-begin', self.__drag_begin_cb)
+        if child.get_realized():
+            child.set_parent_window(self.get_parent_window())
+        child.set_parent(self)
+
+    def __button_release_cb(self, widget, event):
+        if self._dragging:
+            return True
+        else:
+            return False
+
+    def __button_press_cb(self, widget, event):
+        if event.button == 1 and event.type == Gdk.EventType.BUTTON_PRESS:
+            self._last_clicked_icon = widget
+            self._pressed_button = event.button
+            self._press_start_x = event.x
+            self._press_start_y = event.y
+        return False
+
+    def __motion_notify_event_cb(self, widget, event):
+        if not self._dragging_mode:
+            return False
+
+        # if the mouse button is not pressed, no drag should occurr
+        if not event.get_state() & Gdk.ModifierType.BUTTON1_MASK:
+            self._pressed_button = None
+            return False
+
+        if event.is_hint:
+            x, y, state_ = event.window.get_pointer()
+        else:
+            x = event.x
+            y = event.y
+
+        if widget.drag_check_threshold(int(self._press_start_x),
+                                       int(self._press_start_y),
+                                       int(x),
+                                       int(y)):
+            self._dragging = True
+            target_entry = Gtk.TargetEntry.new(*_ICON_DND_TARGET)
+            target_list = Gtk.TargetList.new([target_entry])
+            context_ = widget.drag_begin(target_list,
+                                         Gdk.DragAction.MOVE,
+                                         1,
+                                         event)
+        return False
+
+    def __drag_begin_cb(self, widget, context):
+        if not self._dragging_mode:
+            return False
+
+        pixbuf = GdkPixbuf.Pixbuf.new_from_file(widget.props.file_name)
+
+        self._hot_x = pixbuf.props.width / 2
+        self._hot_y = pixbuf.props.height / 2
+        Gtk.drag_set_icon_pixbuf(context, pixbuf, self._hot_x, self._hot_y)
+
+    def __drag_motion_cb(self, widget, context, x, y, time):
+        if self._last_clicked_icon is not None:
+            Gdk.drag_status(context, context.get_suggested_action(), time)
+            return True
+        else:
+            return False
+
+    def __drag_drop_cb(self, widget, context, x, y, time):
+        if self._last_clicked_icon is not None:
+            target = Gdk.Atom.intern_static_string(_ICON_DND_TARGET[0])
+            self.drag_get_data(context, target, time)
+            self._layout.move_icon(self._last_clicked_icon,
+                                   x - self._hot_x, y - self._hot_y,
+                                   self.get_allocation())
+
+            self._pressed_button = None
+            self._press_start_x = None
+            self._press_start_y = None
+            self._hot_x = None
+            self._hot_y = None
+            self._last_clicked_icon = None
+            self._dragging = False
+
+            return True
+        else:
+            return False
+
+    def __drag_data_received_cb(self, widget, context, x, y, selection_data,
+                                info, time):
+        Gdk.drop_finish(context, success=True, time_=time)
 
     def __connect_to_bundle_registry_cb(self):
         registry = bundleregistry.get_registry()
@@ -141,10 +301,10 @@ class FavoritesView(hippo.Canvas):
         if activity_info.get_bundle_id() == 'org.laptop.JournalActivity':
             return
         icon = ActivityIcon(activity_info)
-        icon.props.size = style.STANDARD_ICON_SIZE
-        icon.set_resume_mode(self._resume_mode)
-        self._box.insert_sorted(icon, 0, self._layout.compare_activities)
-        self._layout.append(icon)
+        icon.props.pixel_size = style.STANDARD_ICON_SIZE
+        #icon.set_resume_mode(self._resume_mode)
+        self.add(icon)
+        icon.show()
 
     def __activity_added_cb(self, activity_registry, activity_info):
         registry = bundleregistry.get_registry()
@@ -152,19 +312,18 @@ class FavoritesView(hippo.Canvas):
                 activity_info.get_activity_version()):
             self._add_activity(activity_info)
 
-    def _find_activity_icon(self, bundle_id, version):
-        for icon in self._box.get_children():
-            if isinstance(icon, ActivityIcon) and \
-                    icon.bundle_id == bundle_id and icon.version == version:
-                return icon
-        return None
-
     def __activity_removed_cb(self, activity_registry, activity_info):
         icon = self._find_activity_icon(activity_info.get_bundle_id(),
                 activity_info.get_activity_version())
         if icon is not None:
-            self._layout.remove(icon)
-            self._box.remove(icon)
+            self.remove(icon)
+
+    def _find_activity_icon(self, bundle_id, version):
+        for icon in self.get_children():
+            if isinstance(icon, ActivityIcon) and \
+                    icon.bundle_id == bundle_id and icon.version == version:
+                return icon
+        return None
 
     def __activity_changed_cb(self, activity_registry, activity_info):
         if activity_info.get_bundle_id() == 'org.laptop.JournalActivity':
@@ -172,163 +331,23 @@ class FavoritesView(hippo.Canvas):
         icon = self._find_activity_icon(activity_info.get_bundle_id(),
                 activity_info.get_activity_version())
         if icon is not None:
-            self._box.remove(icon)
+            self.remove(icon)
 
         registry = bundleregistry.get_registry()
         if registry.is_bundle_favorite(activity_info.get_bundle_id(),
                                        activity_info.get_activity_version()):
             self._add_activity(activity_info)
 
-    def do_size_allocate(self, allocation):
-        width = allocation.width
-        height = allocation.height
-
-        min_w_, my_icon_width = self._my_icon.get_width_request()
-        min_h_, my_icon_height = self._my_icon.get_height_request(
-            my_icon_width)
-        x = (width - my_icon_width) / 2
-        y = (height - my_icon_height - style.GRID_CELL_SIZE) / 2
-        self._layout.move_icon(self._my_icon, x, y, locked=True)
-
-        min_w_, icon_width = self._current_activity.get_width_request()
-        min_h_, icon_height = \
-                self._current_activity.get_height_request(icon_width)
-        x = (width - icon_width) / 2
-        y = (height - my_icon_height - style.GRID_CELL_SIZE) / 2 + \
-                my_icon_height + style.DEFAULT_PADDING
-        self._layout.move_icon(self._current_activity, x, y, locked=True)
-
-        hippo.Canvas.do_size_allocate(self, allocation)
-
-    # TODO: Dnd methods. This should be merged somehow inside hippo-canvas.
-    def __button_press_event_cb(self, widget, event):
-        if event.button == 1 and event.type == gtk.gdk.BUTTON_PRESS:
-            self._last_clicked_icon = self._get_icon_at_coords(event.x,
-                                                               event.y)
-            if self._last_clicked_icon is not None:
-                self._pressed_button = event.button
-                self._press_start_x = event.x
-                self._press_start_y = event.y
-
-        return False
-
-    def _get_icon_at_coords(self, x, y):
-        for icon in self._box.get_children():
-            icon_x, icon_y = icon.get_context().translate_to_widget(icon)
-            icon_width, icon_height = icon.get_allocation()
-
-            if (x >= icon_x) and (x <= icon_x + icon_width) and \
-               (y >= icon_y) and (y <= icon_y + icon_height) and \
-               isinstance(icon, ActivityIcon):
-                return icon
-        return None
-
-    def __motion_notify_event_cb(self, widget, event):
-        if not self._pressed_button:
-            return False
-
-        # if the mouse button is not pressed, no drag should occurr
-        if not event.state & gtk.gdk.BUTTON1_MASK:
-            self._pressed_button = None
-            return False
-
-        if event.is_hint:
-            x, y, state_ = event.window.get_pointer()
-        else:
-            x = event.x
-            y = event.y
-
-        if widget.drag_check_threshold(int(self._press_start_x),
-                                       int(self._press_start_y),
-                                       int(x),
-                                       int(y)):
-            context_ = widget.drag_begin([_ICON_DND_TARGET],
-                                         gtk.gdk.ACTION_MOVE,
-                                         1,
-                                         event)
-        return False
-
-    def __drag_begin_cb(self, widget, context):
-        icon_file_name = self._last_clicked_icon.props.file_name
-        # TODO: we should get the pixbuf from the widget, so it has colors, etc
-        pixbuf = gtk.gdk.pixbuf_new_from_file(icon_file_name)
-
-        self._hot_x = pixbuf.props.width / 2
-        self._hot_y = pixbuf.props.height / 2
-        context.set_icon_pixbuf(pixbuf, self._hot_x, self._hot_y)
-
-    def __drag_motion_cb(self, widget, context, x, y, time):
-        if self._last_clicked_icon is not None:
-            context.drag_status(context.suggested_action, time)
-            return True
-        else:
-            return False
-
-    def __drag_drop_cb(self, widget, context, x, y, time):
-        if self._last_clicked_icon is not None:
-            self.drag_get_data(context, _ICON_DND_TARGET[0])
-
-            self._layout.move_icon(self._last_clicked_icon,
-                                   x - self._hot_x, y - self._hot_y)
-
-            self._pressed_button = None
-            self._press_start_x = None
-            self._press_start_y = None
-            self._hot_x = None
-            self._hot_y = None
-            self._last_clicked_icon = None
-
-            return True
-        else:
-            return False
-
-    def __drag_data_received_cb(self, widget, context, x, y, selection_data,
-                                info, time):
-        context.drop_finish(success=True, time=time)
-
-    def _set_layout(self, layout):
-        if layout not in LAYOUT_MAP:
-            logging.warn('Unknown favorites layout: %r', layout)
-            layout = favoriteslayout.RingLayout.key
-            assert layout in LAYOUT_MAP
-
-        if type(self._layout) == LAYOUT_MAP[layout]:
-            return
-
-        self._layout = LAYOUT_MAP[layout]()
-        self._box.set_layout(self._layout)
-
-        #TODO: compatibility hack while sort() gets added to the hippo python
-        # bindings
-        if hasattr(self._box, 'sort'):
-            self._box.sort(self._layout.compare_activities)
-
-        for icon in self._box.get_children():
-            if icon not in [self._my_icon, self._current_activity]:
-                self._layout.append(icon)
-
-        self._layout.append(self._my_icon, locked=True)
-        self._layout.append(self._current_activity, locked=True)
-
-        if self._layout.allow_dnd():
-            self.drag_source_set(0, [], 0)
-            self.drag_dest_set(0, [], 0)
-        else:
-            self.drag_source_unset()
-            self.drag_dest_unset()
-
-    layout = property(None, _set_layout)
-
-    def add_alert(self, alert):
-        if self._alert is not None:
-            self.remove_alert()
-        alert.set_size_request(gtk.gdk.screen_width(), -1)
-        self._alert = hippo.CanvasWidget(widget=alert)
-        self._box.append(self._alert, hippo.PACK_FIXED)
-
-    def remove_alert(self):
-        self._box.remove(self._alert)
-        self._alert = None
+    def set_filter(self, query):
+        query = query.strip()
+        for icon in self.get_children():
+            if icon not in [self._owner_icon, self._activity_icon]:
+                activity_name = icon.get_activity_name().decode('utf-8')
+                normalized_name = normalize_string(activity_name)
+                if normalized_name.find(query) > -1:
+                    icon.alpha = 1.0
+                else:
+                    icon.alpha = 0.33
 
     def __register_activate_cb(self, icon):
         alert = Alert()
@@ -341,20 +360,20 @@ class FavoritesView(hippo.Canvas):
             alert.props.title = _('Registration Successful')
             alert.props.msg = _('You are now registered ' \
                                 'with your school server.')
-            self._my_icon.set_registered()
+            self._owner_icon.set_registered()
 
         ok_icon = Icon(icon_name='dialog-ok')
-        alert.add_button(gtk.RESPONSE_OK, _('Ok'), ok_icon)
+        alert.add_button(Gtk.ResponseType.OK, _('Ok'), ok_icon)
 
-        self.add_alert(alert)
+        self._box.add_alert(alert)
         alert.connect('response', self.__register_alert_response_cb)
 
     def __register_alert_response_cb(self, alert, response_id):
-        self.remove_alert()
+        self._box.remove_alert()
 
     def set_resume_mode(self, resume_mode):
         self._resume_mode = resume_mode
-        for icon in self._box.get_children():
+        for icon in self.get_children():
             if hasattr(icon, 'set_resume_mode'):
                 icon.set_resume_mode(self._resume_mode)
 
@@ -362,7 +381,7 @@ class FavoritesView(hippo.Canvas):
 class ActivityIcon(CanvasIcon):
     __gtype_name__ = 'SugarFavoriteActivityIcon'
 
-    _BORDER_WIDTH = style.zoom(3)
+    _BORDER_WIDTH = style.zoom(9)
     _MAX_RESUME_ENTRIES = 5
 
     def __init__(self, activity_info):
@@ -371,11 +390,10 @@ class ActivityIcon(CanvasIcon):
 
         self._activity_info = activity_info
         self._journal_entries = []
-        self._hovering = False
         self._resume_mode = True
 
-        self.connect('hovering-changed', self.__hovering_changed_event_cb)
-        self.connect('button-release-event', self.__button_release_event_cb)
+        self.connect_after('button-release-event',
+                           self.__button_release_event_cb)
 
         datastore.updated.connect(self.__datastore_listener_updated_cb)
         datastore.deleted.connect(self.__datastore_listener_deleted_cb)
@@ -435,6 +453,7 @@ class ActivityIcon(CanvasIcon):
         palette = FavoritePalette(self._activity_info, self._journal_entries)
         palette.connect('activate', self.__palette_activate_cb)
         palette.connect('entry-activate', self.__palette_entry_activate_cb)
+        self.connect_to_palette_pop_events(palette)
         return palette
 
     def __palette_activate_cb(self, palette):
@@ -443,45 +462,15 @@ class ActivityIcon(CanvasIcon):
     def __palette_entry_activate_cb(self, palette, metadata):
         self._resume(metadata)
 
-    def __hovering_changed_event_cb(self, icon, hovering):
-        self._hovering = hovering
-        self.emit_paint_needed(0, 0, -1, -1)
-
-    def do_paint_above_children(self, cr, damaged_box):
-        if not self._hovering:
-            return
-
-        width, height = self.get_allocation()
-
-        x = ActivityIcon._BORDER_WIDTH / 2.0
-        y = ActivityIcon._BORDER_WIDTH / 2.0
-        width -= ActivityIcon._BORDER_WIDTH
-        height -= ActivityIcon._BORDER_WIDTH
-        radius = width / 10.0
-
-        cr.move_to(x + radius, y)
-        cr.arc(x + width - radius, y + radius, radius, math.pi * 1.5,
-               math.pi * 2.0)
-        cr.arc(x + width - radius, x + height - radius, radius, 0,
-               math.pi * 0.5)
-        cr.arc(x + radius, y + height - radius, radius, math.pi * 0.5, math.pi)
-        cr.arc(x + radius, y + radius, radius, math.pi, math.pi * 1.5)
-
-        color = style.COLOR_SELECTION_GREY.get_int()
-        hippo.cairo_set_source_rgba32(cr, color)
-        cr.set_line_width(ActivityIcon._BORDER_WIDTH)
-        cr.stroke()
-
-    def do_get_content_height_request(self, for_width):
-        height, height = CanvasIcon.do_get_content_height_request(self,
-                                                                  for_width)
-        height += ActivityIcon._BORDER_WIDTH * 2
-        return height, height
-
-    def do_get_content_width_request(self):
-        width, width = CanvasIcon.do_get_content_width_request(self)
+    def do_get_preferred_width(self):
+        width = CanvasIcon.do_get_preferred_width(self)[0]
         width += ActivityIcon._BORDER_WIDTH * 2
-        return width, width
+        return (width, width)
+
+    def do_get_preferred_height(self):
+        height = CanvasIcon.do_get_preferred_height(self)[0]
+        height += ActivityIcon._BORDER_WIDTH * 2
+        return (height, height)
 
     def __button_release_event_cb(self, icon, event):
         self._activate()
@@ -529,8 +518,8 @@ class FavoritePalette(ActivityPalette):
     __gtype_name__ = 'SugarFavoritePalette'
 
     __gsignals__ = {
-        'entry-activate': (gobject.SIGNAL_RUN_FIRST,
-                           gobject.TYPE_NONE, ([object])),
+        'entry-activate': (GObject.SignalFlags.RUN_FIRST,
+                           None, ([object])),
     }
 
     def __init__(self, activity_info, journal_entries):
@@ -544,40 +533,41 @@ class FavoritePalette(ActivityPalette):
 
         self.props.icon = Icon(file=activity_info.get_icon(),
                                xo_color=xo_color,
-                               icon_size=gtk.ICON_SIZE_LARGE_TOOLBAR)
+                               icon_size=Gtk.IconSize.LARGE_TOOLBAR)
 
         if journal_entries:
             title = journal_entries[0]['title']
-            self.props.secondary_text = glib.markup_escape_text(title)
+            self.props.secondary_text = GLib.markup_escape_text(title)
 
             menu_items = []
             for entry in journal_entries:
                 icon_file_name = misc.get_icon_name(entry)
                 color = misc.get_icon_color(entry)
 
-                menu_item = MenuItem(text_label=entry['title'],
-                                     file_name=icon_file_name,
-                                     xo_color=color)
+                menu_item = PaletteMenuItem(text_label=entry['title'],
+                                            file_name=icon_file_name,
+                                            xo_color=color)
                 menu_item.connect('activate', self.__resume_entry_cb, entry)
                 menu_items.append(menu_item)
                 menu_item.show()
 
             if journal_entries:
-                separator = gtk.SeparatorMenuItem()
+                separator = PaletteMenuItemSeparator()
                 menu_items.append(separator)
                 separator.show()
 
             for i in range(0, len(menu_items)):
-                self.menu.insert(menu_items[i], i)
+                self.menu_box.pack_start(menu_items[i], True, True, 0)
 
     def __resume_entry_cb(self, menu_item, entry):
         if entry is not None:
             self.emit('entry-activate', entry)
 
 
-class CurrentActivityIcon(CanvasIcon, hippo.CanvasItem):
+class CurrentActivityIcon(CanvasIcon):
     def __init__(self):
-        CanvasIcon.__init__(self, cache=True)
+        CanvasIcon.__init__(self, icon_name='activity-journal',
+                            pixel_size=style.STANDARD_ICON_SIZE, cache=True)
         self._home_model = shell.get_model()
         self._home_activity = self._home_model.get_active_activity()
 
@@ -587,16 +577,17 @@ class CurrentActivityIcon(CanvasIcon, hippo.CanvasItem):
         self._home_model.connect('active-activity-changed',
                                  self.__active_activity_changed_cb)
 
-        self.connect('button-release-event', self.__button_release_event_cb)
+        self.connect_after('button-release-event',
+                           self.__button_release_event_cb)
 
     def __button_release_event_cb(self, icon, event):
         window = self._home_model.get_active_activity().get_window()
-        window.activate(gtk.get_current_event_time())
+        window.activate(Gtk.get_current_event_time())
 
     def _update(self):
         self.props.file_name = self._home_activity.get_icon_path()
         self.props.xo_color = self._home_activity.get_icon_color()
-        self.props.size = style.STANDARD_ICON_SIZE
+        self.props.pixel_size = style.STANDARD_ICON_SIZE
 
         if self.palette is not None:
             self.palette.destroy()
@@ -607,6 +598,7 @@ class CurrentActivityIcon(CanvasIcon, hippo.CanvasItem):
             palette = JournalPalette(self._home_activity)
         else:
             palette = CurrentActivityPalette(self._home_activity)
+        self.connect_to_palette_pop_events(palette)
         return palette
 
     def __active_activity_changed_cb(self, home_model, home_activity):
@@ -618,18 +610,27 @@ class OwnerIcon(BuddyIcon):
     __gtype_name__ = 'SugarFavoritesOwnerIcon'
 
     __gsignals__ = {
-        'register-activate': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+        'register-activate': (GObject.SignalFlags.RUN_FIRST, None,
                               ([])),
     }
 
     def __init__(self, size):
-        BuddyIcon.__init__(self, buddy=get_owner_instance(), size=size)
+        BuddyIcon.__init__(self, buddy=get_owner_instance(), pixel_size=size)
 
         self.palette_invoker.cache_palette = True
 
         self._palette_enabled = False
         self._register_menu = None
 
+        # This is a workaround to skip the callback for
+        # enter-notify-event in the parent class the first time.
+        def __enter_notify_event_cb(icon, event):
+            self.unset_state_flags(Gtk.StateFlags.PRELIGHT)
+            self.disconnect(self._enter_notify_hid)
+
+        self._enter_notify_hid = self.connect('enter-notify-event',
+                                              __enter_notify_event_cb)
+
     def create_palette(self):
         if not self._palette_enabled:
             self._palette_enabled = True
@@ -637,32 +638,33 @@ class OwnerIcon(BuddyIcon):
 
         palette = BuddyMenu(get_owner_instance())
 
-        client = gconf.client_get_default()
+        client = GConf.Client.get_default()
         backup_url = client.get_string('/desktop/sugar/backup_url')
 
         if not backup_url:
-            self._register_menu = MenuItem(_('Register'), 'media-record')
+            self._register_menu = PaletteMenuItem(_('Register'),
+                                                  'media-record')
         else:
-            self._register_menu = MenuItem(_('Register again'),
-                                           'media-record')
+            self._register_menu = PaletteMenuItem(_('Register again'),
+                                                  'media-record')
 
         self._register_menu.connect('activate', self.__register_activate_cb)
-        palette.menu.append(self._register_menu)
+        palette.menu_box.pack_end(self._register_menu, True, True, 0)
         self._register_menu.show()
 
-        return palette
+        self.connect_to_palette_pop_events(palette)
 
-    def get_toplevel(self):
-        return hippo.get_canvas_for_item(self).get_toplevel()
+        return palette
 
     def __register_activate_cb(self, menuitem):
         self.emit('register-activate')
 
     def set_registered(self):
-        self.palette.menu.remove(self._register_menu)
-        self._register_menu = MenuItem(_('Register again'), 'media-record')
+        self.palette.menu_box.remove(self._register_menu)
+        self._register_menu = PaletteMenuItem(_('Register again'),
+                                              'media-record')
         self._register_menu.connect('activate', self.__register_activate_cb)
-        self.palette.menu.append(self._register_menu)
+        self.palette.menu_box.pack_end(self._register_menu, True, True, 0)
         self._register_menu.show()
 
 
@@ -671,7 +673,7 @@ class FavoritesSetting(object):
     _FAVORITES_KEY = '/desktop/sugar/desktop/favorites_layout'
 
     def __init__(self):
-        client = gconf.client_get_default()
+        client = GConf.Client.get_default()
         self._layout = client.get_string(self._FAVORITES_KEY)
         logging.debug('FavoritesSetting layout %r', self._layout)
 
@@ -687,7 +689,7 @@ class FavoritesSetting(object):
         if layout != self._layout:
             self._layout = layout
 
-            client = gconf.client_get_default()
+            client = GConf.Client.get_default()
             client.set_string(self._FAVORITES_KEY, layout)
 
             self.changed.send(self)
