@@ -1,13 +1,11 @@
 import os
 
-import gtk
+from sugar3 import env
+from sugar3.profile import get_profile
 
-from sugar import env
-from sugar.profile import get_profile
-
-from jarabe.intro.window import IntroWindow
 from jarabe.intro.window import create_profile
 
+
 def check_profile():
     profile = get_profile()
 
@@ -19,7 +17,4 @@ def check_profile():
     if os.path.exists(path):
         profile.convert_profile()
 
-    if not profile.is_valid():
-        win = IntroWindow()
-        win.show_all()
-        gtk.main()
+    return profile.is_valid()
