@@ -17,13 +17,13 @@
 import logging
 
 import simplejson
-import gobject
-import gtk
+from gi.repository import GObject
+from gi.repository import Gtk
 from gettext import gettext as _
 
-from sugar.graphics.xocolor import XoColor
-from sugar.graphics import style
-from sugar import util
+from sugar3.graphics.xocolor import XoColor
+from sugar3.graphics import style
+from sugar3 import util
 
 from jarabe.journal import model
 from jarabe.journal import misc
@@ -34,12 +34,12 @@ DS_DBUS_INTERFACE = 'org.laptop.sugar.DataStore'
 DS_DBUS_PATH = '/org/laptop/sugar/DataStore'
 
 
-class ListModel(gtk.GenericTreeModel, gtk.TreeDragSource):
+class ListModel(GObject.GObject, Gtk.TreeModel, Gtk.TreeDragSource):
     __gtype_name__ = 'JournalListModel'
 
     __gsignals__ = {
-        'ready': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([])),
-        'progress': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ([])),
+        'ready': (GObject.SignalFlags.RUN_FIRST, None, ([])),
+        'progress': (GObject.SignalFlags.RUN_FIRST, None, ([])),
     }
 
     COLUMN_UID = 0
@@ -73,7 +73,7 @@ class ListModel(gtk.GenericTreeModel, gtk.TreeDragSource):
     _PAGE_SIZE = 10
 
     def __init__(self, query):
-        gobject.GObject.__init__(self)
+        GObject.GObject.__init__(self)
 
         self._last_requested_index = None
         self._cached_row = None
@@ -102,22 +102,23 @@ class ListModel(gtk.GenericTreeModel, gtk.TreeDragSource):
     def get_metadata(self, path):
         return model.get(self[path][ListModel.COLUMN_UID])
 
-    def on_get_n_columns(self):
+    def do_get_n_columns(self):
         return len(ListModel._COLUMN_TYPES)
 
-    def on_get_column_type(self, index):
+    def do_get_column_type(self, index):
         return ListModel._COLUMN_TYPES[index]
 
-    def on_iter_n_children(self, iterator):
+    def do_iter_n_children(self, iterator):
         if iterator == None:
             return self._result_set.length
         else:
             return 0
 
-    def on_get_value(self, index, column):
+    def do_get_value(self, iterator, column):
         if self.view_is_resizing:
             return None
 
+        index = iterator.user_data
         if index == self._last_requested_index:
             return self._cached_row[column]
 
@@ -140,7 +141,7 @@ class ListModel(gtk.GenericTreeModel, gtk.TreeDragSource):
             xo_color = misc.get_icon_color(metadata)
         self._cached_row.append(xo_color)
 
-        title = gobject.markup_escape_text(metadata.get('title',
+        title = GObject.markup_escape_text(metadata.get('title',
                                            _('Untitled')))
         self._cached_row.append('<b>%s</b>' % (title, ))
 
@@ -200,44 +201,53 @@ class ListModel(gtk.GenericTreeModel, gtk.TreeDragSource):
 
         return self._cached_row[column]
 
-    def on_iter_nth_child(self, iterator, n):
-        return n
+    def do_iter_nth_child(self, parent_iter, n):
+        return (False, None)
 
-    def on_get_path(self, iterator):
-        return (iterator)
+    def do_get_path(self, iterator):
+        treepath = Gtk.TreePath((iterator.user_data,))
+        return treepath
 
-    def on_get_iter(self, path):
-        return path[0]
+    def do_get_iter(self, path):
+        idx = path.get_indices()[0]
+        iterator = Gtk.TreeIter()
+        iterator.user_data = idx
+        return (True, iterator)
 
-    def on_iter_next(self, iterator):
-        if iterator != None:
-            if iterator >= self._result_set.length - 1:
-                return None
-            return iterator + 1
-        return None
+    def do_iter_next(self, iterator):
+        idx = iterator.user_data + 1
+        if idx >= self._result_set.length:
+            iterator.stamp = -1
+            return (False, iterator)
+        else:
+            iterator.user_data = idx
+            return (True, iterator)
 
-    def on_get_flags(self):
-        return gtk.TREE_MODEL_ITERS_PERSIST | gtk.TREE_MODEL_LIST_ONLY
+    def do_get_flags(self):
+        return Gtk.TreeModelFlags.ITERS_PERSIST | Gtk.TreeModelFlags.LIST_ONLY
 
-    def on_iter_children(self, iterator):
-        return None
+    def do_iter_children(self, iterator):
+        return (False, iterator)
 
-    def on_iter_has_child(self, iterator):
+    def do_iter_has_child(self, iterator):
         return False
 
-    def on_iter_parent(self, iterator):
-        return None
+    def do_iter_parent(self, iterator):
+        return (False, Gtk.TreeIter())
 
     def do_drag_data_get(self, path, selection):
         uid = self[path][ListModel.COLUMN_UID]
-        if selection.target == 'text/uri-list':
+        target_atom = selection.get_target()
+        target_name = target_atom.name()
+        if target_name == 'text/uri-list':
             # Get hold of a reference so the temp file doesn't get deleted
             self._temp_drag_file_path = model.get_file(uid)
             logging.debug('putting %r in selection', self._temp_drag_file_path)
-            selection.set(selection.target, 8, self._temp_drag_file_path)
+            selection.set(target_atom, 8, self._temp_drag_file_path)
             return True
-        elif selection.target == 'journal-object-id':
-            selection.set(selection.target, 8, uid)
+        elif target_name == 'journal-object-id':
+            # uid is unicode but Gtk.SelectionData.set() needs str
+            selection.set(target_atom, 8, str(uid))
             return True
 
         return False
