@@ -21,17 +21,17 @@
 from gettext import gettext as _
 import logging
 import os
+import uuid
 
 import dbus
 import dbus.service
-import gobject
+from gi.repository import GObject
 import ConfigParser
-import gconf
+from gi.repository import GConf
 import ctypes
 
-from sugar import dispatch
-from sugar import env
-from sugar.util import unique_id
+from sugar3 import dispatch
+from sugar3 import env
 
 NM_STATE_UNKNOWN = 0
 NM_STATE_ASLEEP = 10
@@ -595,14 +595,14 @@ class SecretAgent(dbus.service.Object):
         self.secrets_request.send(self, settings=settings, response=response)
 
 
-class AccessPoint(gobject.GObject):
+class AccessPoint(GObject.GObject):
     __gsignals__ = {
-        'props-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
-                          ([gobject.TYPE_PYOBJECT])),
+        'props-changed': (GObject.SignalFlags.RUN_FIRST, None,
+                          ([GObject.TYPE_PYOBJECT])),
     }
 
     def __init__(self, device, model):
-        self.__gobject_init__()
+        GObject.GObject.__init__(self)
         self.device = device
         self.model = model
 
@@ -744,13 +744,13 @@ def _add_and_activate_error_cb(err):
     logging.error('Failed to add and activate connection: %s', err)
 
 
-class Connection(gobject.GObject):
+class Connection(GObject.GObject):
     __gsignals__ = {
-        'removed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
+        'removed': (GObject.SignalFlags.RUN_LAST, None, ()),
     }
 
     def __init__(self, bus, path):
-        gobject.GObject.__init__(self)
+        GObject.GObject.__init__(self)
         obj = bus.get_object(NM_SERVICE, path)
         self._connection = dbus.Interface(obj, NM_CONNECTION_IFACE)
         self._removed_handle = self._connection.connect_to_signal(
@@ -991,7 +991,7 @@ def create_gsm_connection(username, password, number, apn, pin):
 
     settings.connection.id = GSM_CONNECTION_ID
     settings.connection.type = NM_CONNECTION_TYPE_GSM
-    settings.connection.uuid = unique_id()
+    settings.connection.uuid = str(uuid.uuid4())
     settings.connection.autoconnect = False
     settings.ip4_config.method = 'auto'
     settings.serial.baud = GSM_BAUD_RATE
@@ -1004,7 +1004,7 @@ def _migrate_old_gsm_connection():
         # don't attempt migration if a NM-level connection already exists
         return
 
-    client = gconf.client_get_default()
+    client = GConf.Client.get_default()
 
     username = client.get_string(GSM_USERNAME_PATH) or ''
     password = client.get_string(GSM_PASSWORD_PATH) or ''
